Package: snapcount
Type: Package
Title: R/Bioconductor Package for interfacing with Snaptron for rapid
        querying of expression counts
Version: 1.23.0
Authors@R: 
    c(person(given = "Rone",
             family = "Charles",
             role = c("aut", "cre"),
             email = "rcharle8@jh.edu"))
Description: snapcount is a client interface to the Snaptron webservices
    which support querying by gene name or genomic region.
    Results include raw expression counts derived from alignment of RNA-seq samples
    and/or various summarized measures of expression across one or more regions/genes
    per-sample (e.g. percent spliced in).
Depends: R (>= 4.0.0)
Imports: R6, httr, rlang, purrr, jsonlite, assertthat, data.table,
        Matrix, magrittr, methods, stringr, stats, IRanges,
        GenomicRanges, SummarizedExperiment
Suggests: BiocManager, bit64, covr, knitcitations, knitr (>= 1.6),
        devtools, BiocStyle (>= 2.5.19), rmarkdown (>= 0.9.5), testthat
        (>= 2.1.0)
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.1.1
Roxygen: list(markdown = TRUE, roclets = c("rd", "namespace",
        "collate"))
URL: https://github.com/langmead-lab/snapcount
BugReports: https://github.com/langmead-lab/snapcount/issues
biocViews: Coverage, GeneExpression, RNASeq, Sequencing, Software,
        DataImport
VignetteBuilder: knitr
ByteCompile: true
git_url: https://git.bioconductor.org/packages/snapcount
git_branch: devel
git_last_commit: 1af1119
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-31
NeedsCompilation: no
Packaged: 2025-11-01 00:18:29 UTC; biocbuild
Author: Rone Charles [aut, cre]
Maintainer: Rone Charles <rcharle8@jh.edu>
