# Generated by roxygen2: do not edit by hand

export(Compilation)
export(Coordinates)
export(QueryBuilder)
export(from_url)
export(get_column_filters)
export(get_compilation)
export(get_coordinate_modifier)
export(get_regions)
export(get_row_filters)
export(get_sids)
export(junction_inclusion_ratio)
export(junction_intersection)
export(junction_union)
export(percent_spliced_in)
export(query_exon)
export(query_gene)
export(query_jx)
export(set_column_filters)
export(set_compilation)
export(set_coordinate_modifier)
export(set_regions)
export(set_row_filters)
export(set_sids)
export(shared_sample_counts)
export(tissue_specificity)
export(uri_of_last_successful_request)
import(data.table)
importFrom(GenomicRanges,GRanges)
importFrom(R6,R6Class)
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(assertthat,assert_that)
importFrom(magrittr,"%>%")
importFrom(methods,is)
importFrom(rlang,"!!")
importFrom(stats,median)
